function [ind_Q_vec,ind_S_vec,QS,QS_std,R_amp_m,R_amp_std] = compute_qrs_size(ecg,jqrs,fs,debug)
% This function computes the QS interval and R peak amplitude. It only
% takes into account these interval/hight in the interval of good quality.
%
% inputs
%   ecg: raw ecg
%   jqrs: jqrs R-peak annotations
%   sqi: second by second signal quality
%   fs: sampling frequency
%
% outputs
%   ind_Q_vec: index of Q fiducials
%   ind_S_vec: index of S fiducials
%   QS: median QS interval
%   QS_std: standard deviation on QS intervals
%   R_amp: median R peak amplitude
%   R_amp:_std: std on R peak amplitude
%
% Copyright (C) 2017 Joachim A. Behar
% Bio-electric and Bio-energetic system Group, Technion-IIT, Israel
% joachim.a.behar@gmail.com
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 2 of the License, or (at your
% option) any later version.
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.

delta_s = 0.150*fs;
NB_QRS = length(jqrs);

% == prefiltering
LOW_CUT_FREQ = 2;
HIGH_CUT_FREQ = 45;
[b_lp,a_lp] = butter(5,HIGH_CUT_FREQ/(fs/2),'high');
[b_bas,a_bas] = butter(2,LOW_CUT_FREQ/(fs/2),'high');
ecg = ecg-mean(ecg);
bpfecg = ecg'-filtfilt(b_lp,a_lp,ecg');
bpfecg = filtfilt(b_bas,a_bas,bpfecg);

if sum(sign(bpfecg(jqrs)))<0
    bpfecg = -bpfecg;
end

ind_Q_vec = zeros(NB_QRS,1);
ind_S_vec = zeros(NB_QRS,1);
R_amp = zeros(NB_QRS,1);

% = loop over each R-peak
for kk=1:NB_QRS
    
    if jqrs(kk)-delta_s>0 && jqrs(kk)+delta_s<length(ecg)
        
        % check border are okay and that the sqi is 1
        %[~,ind_Q] = min(bpfecg(jqrs(kk)-delta_s:jqrs(kk)));
        %[~,ind_S] = min(bpfecg(jqrs(kk):jqrs(kk)+delta_s));
        %ind_Q_vec(kk) = jqrs(kk)-delta_s+ind_Q-1;
        %ind_S_vec(kk) = jqrs(kk)+ind_S-1;        
        
        [~,ind_Q] = findpeaks(-bpfecg(jqrs(kk)-delta_s:jqrs(kk)));
        [~,ind_S] = findpeaks(-bpfecg(jqrs(kk):jqrs(kk)+delta_s));
        ind_Q(delta_s-ind_Q<fs*0.02) = [];
        ind_S(ind_S<fs*0.02) = [];
        
        R_amp(kk) = ecg(jqrs(kk));
        if isempty(ind_Q) || isempty(ind_S)
            ind_Q_vec(kk) = NaN;
            ind_S_vec(kk) = NaN;
        else
            ind_Q_vec(kk) = jqrs(kk)-delta_s+ind_Q(end)-1;
            ind_S_vec(kk) = jqrs(kk)+ind_S(1)-1;
        end

    else
        ind_Q_vec(kk) = NaN;
        ind_S_vec(kk) = NaN;
    end
end

QS = nanmedian(ind_S_vec-ind_Q_vec)/fs;
QS_std = nanstd(ind_S_vec-ind_Q_vec)/fs;
R_amp_m = nanmedian(R_amp);
R_amp_std = nanstd(R_amp);

if debug
    plot(ecg);
    hold on; plot(bpfecg,'--r');
    hold on; plot(jqrs,ecg(jqrs),'+r')
    hold on; plot(ind_Q_vec(~isnan(ind_Q_vec)),ecg(ind_Q_vec(~isnan(ind_Q_vec))),'+g');
    hold on; plot(ind_S_vec(~isnan(ind_S_vec)),ecg(ind_S_vec(~isnan(ind_S_vec))),'+k');
    legend('ecg','filtered ecg','Q','S');
end
end